/* subtrain.c
 *
 * Generative training an HMM from unaligned sequences
 * (not using negative examples).
 *
 * This module is based on HMMER 1.8.5 source file train_main.c.
 */

#include "config.h"             /* compile-time configuration constants */
#include "squidconf.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include <float.h>

#include "structs.h"            /* data structures, macros, #define's   */
#include "funcs.h"              /* function declarations                */
#include "squid.h"              /* general sequence analysis library    */
#include "subloc.h"             /* function declarations for SUBLOC     */

/* Function: P7Distance()
 *
 * Purpose:  Test two models for how different they are, using
 *           a simple squared difference measure on all homologous
 *           parameters. They must have the same architecture:
 *           i.e. check that newhmm->M == oldhmm->M before calling.
 *
 * Args:     newhmm  - new HMM, probability form
 *           oldhmm  - old HMM, probability form
 *
 * Return:   distance.
 */
static float
P7Distance(struct plan7_s *newhmm, struct plan7_s *oldhmm)
{
  int    k, x, ts;
  float  distance = 0.0;

  Prob2Score(.5, 1.0);

  for (k = 1; k < newhmm->M; k++) /* main model transitions */
    for (ts = 0; ts < 7; ts++)
      distance += SQR( 100. * (newhmm->tsc[ts][k] - oldhmm->tsc[ts][k]));
  for (k = 1; k <= newhmm->M; k++) /* match emissions */
    for (x = 0; x < Alphabet_size; x++)
      distance += SQR( 100. * (newhmm->msc[x][k] - oldhmm->msc[x][k]));
  for (k = 1; k < newhmm->M; k++) /* insert emissions  */
    for (x = 0; x < Alphabet_size; x++)
      distance += SQR( 100. * (newhmm->isc[x][k] - oldhmm->isc[x][k]));

  distance = sqrt(distance) / newhmm->M;
  return distance;
}
